% File used in combination with TYPE 155 "Calling Matlab" in TRNSYS 
% -----------------------------------------------------------------------
mFileErrorCode = 100;       % Beginning of the m-file.

load('propTabAir.mat')      
load('propTabAir2.mat')
% These loaded applications are made by Stefan Billig. Used to interpolate thermodynamical properties from the puplished values in WC Reynolds "Thermodynamics Properties in SI".

% Process Inputs and global parameters
% -----------------------------------------------------------------------
T_f_i_C = trnInputs(1);     % inlet temperature [C]
T_amb_C = trnInputs(2);     % ambient temperature [C]
T_sky_C = trnInputs(3);     % sky temperature [C]
T_3_inp_C = trnInputs(4);   % back surface temperature [C]
G_T = trnInputs(5);         % incident solar radiation [kJ/hr*m^2]
slope = trnInputs(6);       % collector slope [degrees]
h_conv_top = trnInputs(7);  % top heat loss coefficient [kJ/hr*m^2*K]
atmP = trnInputs(8);        % atmospheric pressure [atm]
L = trnInputs(9);           % collector length [m]
W = trnInputs(10);          % collector width [m]
alpha_PV = trnInputs(11);   % absorbtance of the PV surface 
em_PV = trnInputs(12);      % emissivity PV
R_2 = trnInputs(13);        % susbstrate resistance [h*m^2*K/kJ]
em_1 = trnInputs(14);       % channel emissivity - top
em_2 = trnInputs(15);       % channel emissivity - bottom
R_3 = trnInputs(16);        % back resistance [h*m^2*K/kJ
chanHeight = trnInputs(17); % channel height [m]
n_nom = trnInputs(18);      % reference PV efficiency
T_ref_C = trnInputs(19);    % reference temperature[C]
G_ref = trnInputs(20);      % reference radiation [kJ/hr*m^2]
Eff_T = trnInputs(21);      % temperature efficiency modifier [1/C]
Eff_G = trnInputs(22);      % radiation efficiency modifier [h*m^2/kJ]


mFileErrorCode = 110;       % After processing inputs

%Other parameters
%----------------------------------------------------------------------
T_amb = T_amb_C + 273.15;

SB = 5.670373*10^(-8);      %Stefan Boltzmann coefficient
area = W*L;
D_h = (4*W*chanHeight)/(2*W + 2*chanHeight); %Hydraulic diameter

g = 9.81;
c_in = 1.5;
c_o = 1;
f = 0.056;

T_ref = T_ref_C + 273.15;   % Temperatures changed to Kelvin
T_3_inp = T_3_inp_C + 273.15;   
T_sky = T_sky_C + 273.15;
T_f_i = T_f_i_C + 273.15;

mFileErrorCode = 150;       % Beginning of iterative call

%Step 1: Initial guess of values
%----------------------------------------------------------------------
T_mean_f_n = T_f_i;
T_PV_n = T_amb;
T_1_n = T_f_i;
T_2_n = T_f_i;
T_f_o = T_amb+10;

%Iteration check parameters
diffT_f = 1;
diffT_s1 = 1;
diffT_s2 = 1;
diffT_PV = 1;
diff_m_dot = 1;
it_counter = 0;

%% 
tolerance=1e-3;
while abs(diffT_PV) > tolerance || abs(diff_m_dot) > tolerance || abs(diffT_s1)> tolerance || abs(diffT_s2) > tolerance || abs(diffT_PV) > tolerance || abs(diff_m_dot) > tolerance

    if it_counter > 100         %number of iterations accepted until default value is returned
    return
    end
        
T_mean_f = T_mean_f_n;
T_PV = T_PV_n;
T_1 = T_1_n;
T_2 = T_2_n;    
    
it_counter = it_counter +1;     %iteration counter

mu=airProp2(T_mean_f,'my');     %[kg/m*s]
C_p=airProp2(T_mean_f,'cp');    %[J/Kg*K]
k_fluid=airProp2(T_mean_f,'k'); %[W/m*K]
rho=airProp2(T_mean_f,'rho');   %[kg/m^3]
Pr = airProp2(T_mean_f,'Pr');   %[-]
alpha_f = airProp2(T_mean_f,'alpha'); %m^2/s
ny = airProp2(T_mean_f,'ny');   %m^2/s


%units SI->Trnsys
mu=mu*3600;                     %[kg/m*hr]
k_fluid = k_fluid*3.6;          %[kg/hr*m*K]
C_p = C_p/1000;                 %[kJ/kg*K]

if T_f_o > T_amb                %calculating natural air flow
    m_dot_i = rho * (W*chanHeight/area) * sqrt((2*g*L*sind(slope)*(T_f_o - T_amb)) / (T_amb*(c_in+c_o+f*(L/D_h)))) * area * 3600;
else 
    m_dot_i = 0;
end

mFileErrorCode = 160;

%Step 2: Calculates the radiation heat transfer coefficients using
%----------------------------------------------------------------------
h_rad_top = (em_PV * SB *(T_PV + T_sky)*(T_PV^2 + T_sky^2))*3.6;  %[kj/hr*m^2*K]

h_rad_1to2 = ((SB*(T_1^2 + T_2^2) * (T_1 + T_2)) / ((1/em_1) + (1/em_2) - 1))*3.6; %[kj/hr*m^2*K] 

%Step 3: Calculate the PV cell efficiency and S
%----------------------------------------------------------------------
X_cellTemp = 1 + Eff_T*(T_PV - T_ref); 
X_radiation = 1 + Eff_G*(G_T - G_ref); 

n_PV = max(0,n_nom * X_cellTemp * X_radiation);

S = alpha_PV * G_T * (1 - n_PV);

mFileErrorCode = 170;

%Step 4: Calculate the fluid heat transfer coefficient
%----------------------------------------------------------------------
Re = (4*m_dot_i)/ (pi * D_h * mu);

if Re <= 0
    Ra = max(1,((g*(T_PV - T_3_inp)*chanHeight^3)/(((T_PV + T_3_inp)/2)*ny*alpha_f)));
    Nu = 1 + 1.44*(1-1708*(sind(1.8*slope)^1.6)/(Ra*cosd(slope)))*max(0,1-1708/(Ra*cosd(slope))) + max(0, ((Ra*cosd(slope)/5830)^(1/3)-1)); 
elseif Re < 2300 
    Nu = 3.66;
else
    T_surf =(T_1+T_2)/2;
    if T_surf >= T_mean_f
        N=0.4;
    else
        N=0.3;
    end
    Nu = 0.023*Re^(0.8)*Pr^(N); %Turbulent flow
end

h_fluid = (Nu * k_fluid)/D_h;   %[kJ/hr*m^2*K]

mFileErrorCode = 180;

%Step 5: Calculate the fluid outlet temperature using and the mean fluid temperature
%----------------------------------------------------------------------
F = h_rad_top*R_2 + h_conv_top * R_2 + 1;   %[-]
j = h_fluid + h_rad_1to2 + (1/R_3);         %[kJ/hr*m^2*K]
m = (1/R_2) - 1/(R_2 *F) + h_fluid + h_rad_1to2 - ((h_rad_1to2)^2 / j);

a = (h_fluid^2 / m) + (2*h_fluid^2*h_rad_1to2)/(m*j) - 2*h_fluid + (h_fluid^2/j)+(h_fluid^2*h_rad_1to2^2)/(m*j^2); %[kJ/hr*m^2*K]

b = (h_fluid*S)/(F*m) + (h_fluid*h_conv_top*T_amb)/(F*m) + (h_fluid*h_rad_top*T_sky)/(F*m) + (h_fluid*h_rad_1to2*T_3_inp)/(R_3*j*m) + (h_fluid*h_rad_1to2*S)/(F*j*m) + (h_fluid*h_rad_1to2*h_conv_top*T_amb)/(F*j*m)+ (h_fluid*h_rad_1to2*h_rad_top*T_sky)/(F*j*m) + (h_fluid*h_rad_1to2^2*T_3_inp)/(R_3*j^2*m) + (h_fluid*T_3_inp)/(R_3*j);

if m_dot_i <= 0
    Q_u = 0;
    T_mean_f_n = -b/a;
    T_f_o = T_mean_f_n;  
else
    T_f_o = ((T_f_i) + (b/a))* exp((area*a)/(m_dot_i *C_p )) - b/a;
    Q_u = m_dot_i *C_p * (T_f_o - T_f_i);
    T_mean_f_n = ((T_f_i + (b/a)) / ((area*a)/(m_dot_i*C_p)))*exp((area*a)/(m_dot_i*C_p)) - (T_f_i + (b/a))/ ((area*a)/(m_dot_i*C_p)) - b/a; %568.31
end

%Step 6: Calculate the mean surface temperature
%----------------------------------------------------------------------
T_1_n = S/(m*F) + (h_conv_top*T_amb)/(m*F) + (h_rad_top*T_sky)/(m*F)+(h_fluid*T_mean_f_n)/m + (h_rad_1to2*h_fluid*T_mean_f_n)/(m*j) + (h_rad_1to2*T_3_inp)/(m*j*R_3);

T_2_n = (h_rad_1to2*T_1_n)/j + (h_fluid*T_mean_f_n)/j + T_3_inp/(j*R_3);

T_PV_n = (R_2*S)/F + (R_2*h_conv_top*T_amb)/F + (R_2*h_rad_top*T_sky)/F + T_1_n/F;

diffT_f = T_mean_f_n - T_mean_f;
diffT_s1 = T_1_n - T_1;
diffT_s2 = T_2_n - T_2;
diffT_PV = T_PV_n - T_PV;


if T_f_o > T_amb
    m_dot_i_n = rho * (W*chanHeight/area) * sqrt((2*g*L*sind(slope)*(T_f_o - T_amb)) / (T_amb*(c_in+c_o+f*(L/D_h)))) * area * 3600;
else 
    m_dot_i_n = 0;
end
    
diff_m_dot = m_dot_i - m_dot_i_n;

% --- Set outputs ---
if G_T == 0
    n_th = 0;
else n_th = Q_u / (area*G_T); 
end

power = area * alpha_PV *G_T *n_PV;

if  h_conv_top *area * (T_PV_n - T_amb) < 0
    Q_loss_top_conv = 0;
else Q_loss_top_conv = h_conv_top *area * (T_PV_n - T_amb);
end
    
Q_loss_top_rad = h_rad_top * area * (T_PV_n - T_sky);

Q_abs = area * alpha_PV *G_T * n_PV;

Q_loss_back = Q_abs + power - Q_u - Q_loss_top_rad - Q_loss_top_conv;

trnOutputs(1) = T_f_o - 273.15;
trnOutputs(2) = m_dot_i;
trnOutputs(3) = Q_u; 
trnOutputs(4) = n_th;
trnOutputs(5) = power;
trnOutputs(6) = n_PV;
trnOutputs(8) = T_1_n-273.15;
trnOutputs(9) = T_mean_f_n-273.15;
trnOutputs(10) = T_2_n-273.15;
trnOutputs(11) = T_3_inp-273.15;
trnOutputs(12) = Q_loss_top_conv;
trnOutputs(13) = Q_loss_top_rad;
trnOutputs(14) = Q_loss_back;
trnOutputs(15) = Q_abs;
trnOutputs(16) = it_counter;
end

mFileErrorCode = 0; % Reached the end of the m-file without errors
return